#include <AIV/Trades.mqh>
#include <AIV/Deals.mqh>

input int InpLevel = 30;
input int InpPeriod = 15;

input int InpDaysDeal = 15;
input int InpDaysUpdate = 1;
input double InpStop = 100;
input double InpProfit = 100;
input double InpVolume = 0.01;
input double InpProbability = 60;
input double InpFactor = 1;

struct SGenerics
{
    int iRsi;
    int day;
    int start;
    int index;
    SBets bets[];
    SDeals deals[]; 
    SSeries series[];
};

struct SSpecials
{
    double iBalance, iBet, iProcess, iVolume;
};

SGenerics iG;
SSpecials iR, iV;
SOptions iOptions;

int OnInit()
{
    ZeroMemory(iG);
    ZeroMemory(iV);
    ZeroMemory(iR);
    
    iR.iBalance = AccountInfoDouble(ACCOUNT_BALANCE);
    iV.iBalance = GetBalanceVirtual(iOptions, 1);
    iG.iRsi = iRSI(_Symbol, _Period, InpPeriod, PRICE_OPEN);
    iR.iVolume = iV.iVolume = InpVolume;
    
    iOptions.mBalance = iR.iBalance;
    iOptions.mContract = SymbolInfoDouble(_Symbol, SYMBOL_TRADE_CONTRACT_SIZE);
    iOptions.mLeverage = double(AccountInfoInteger(ACCOUNT_LEVERAGE));
    iOptions.mCurrency = AccountInfoString(ACCOUNT_CURRENCY);

    MqlDateTime now = {0};
    TimeToStruct(GetSingleTime(_Symbol, _Period), now);
    
    iG.day = now.day_of_year;
    
    return(0);
}

void OnDeinit(const int e)
{
    UpdateBets(1);
}

void UpdateBets(const int track = 0)
{
    GetDeals(iG.deals, DEAL_ENTRY_OUT);
    //string o1 = LogSeries(iG.deals, iG.series);
    string o1 = LogSeriesVirtual(iOptions, iG.series, PeriodSeconds(PERIOD_D1) * InpDaysDeal);
    string o2 = LogRange(iG.series, iG.bets);

    if (track)
    {
        LogData(o1);
        LogData(o2);
    }
}

void OnTick()
{
    PositionControlVirtual(iOptions, _Symbol);

    int direction = CanTrade();
    MqlDateTime now = {0};
    TimeToStruct(GetSingleTime(_Symbol, _Period), now);
    
    if (iG.day != now.day_of_year && MathAbs(iG.day - now.day_of_year) > InpDaysUpdate)
    {
        UpdateBets();
        iG.day = now.day_of_year;
        iG.start = 1;
    }
    
    if (PositionTotalVirtual(iOptions) == 0 && direction != 0)
    {
        double last = GetBalanceVirtual(iOptions, 1);
        iG.index = last < iV.iBalance ? iG.index + 1 : 0;
        iV.iBalance = last;
        PositionOpenRelativeVirtual(iOptions, _Symbol, direction, InpVolume, InpStop * _Point, InpProfit * _Point);
    }
    
    bool C1 = ArraySize(iG.bets) > iG.index && iG.bets[iG.index].mPlus * 100 / iG.bets[iG.index].mCount > InpProbability;
    
    if (iG.start && PositionsTotal() == 0 && direction != 0 && C1)
    {
        double last = AccountInfoDouble(ACCOUNT_BALANCE);
        iR.iVolume = last > iR.iBalance ? InpVolume : iR.iVolume * InpFactor;
        iR.iBalance = last;
        PositionOpenRelative(_Symbol, direction, iR.iVolume, InpStop * _Point, InpProfit * _Point);
    }
}

int CanTrade()
{
    double points[];
    CopyBuffer(iG.iRsi, 0, 0, 2, points);

    bool C1 = points[1] > points[0] && (points[1] < InpLevel || points[0] < InpLevel);
    bool C2 = points[1] < points[0] && (points[1] > 100 - InpLevel || points[0] > 100 - InpLevel);

    if (C1 || C2) 
    {
        return(C1 ? 1 : -1);
    }
    
    return(0);
}